# -*- coding: utf-8 -*-
import cgi


PAGE_HTML = """
<!DOCTYPE html>
<html>
    <head>
    <title>pyinfo()</title>
    <meta name="robots" content="noindex,nofollow,noarchive,nosnippet">
    {style}
    </head>
    <body>
        <div class="center">
        {h.section_title}

        <h2>System</h2>
        {h.section_system}

        <h2>Python Internals</h2>
        {h.section_py_internals}

        <h2>OS Internals</h2>
        {h.section_os_internals}

        <h2>WSGI Environment</h2>
        {h.section_environ}

        <h2>Database support</h2>
        {h.section_database}

        <h2>Compression and archiving</h2>
        {h.section_compression}

        {h.section_ldap}

        {h.section_socket}

        <h2>Multimedia support</h2>
        {h.section_multimedia}

        <h2>Copyright</h2>
        {h.section_copyright}

        </div>
    </body>
</html>
"""

PAGE_STYLE = """
    <style type="text/css">
     body{background-color:#fff;color:#000}
     body,td,th,h1,h2{font-family:sans-serif}
     pre{margin:0px;font-family:monospace}
     a:link{color:#009;text-decoration:none;background-color:#fff}
     a:hover{text-decoration:underline}
     table{border-collapse:collapse}
     .center{text-align:center}
     .center table{margin-left:auto;margin-right:auto;text-align:left}
     .center th{text-align:center !important}
     td,th{border:1px solid #999999;font-size:75%;vertical-align:baseline}
     h1{font-size:150%}
     h2{font-size:125%}
     .p{text-align:left}
     .e{width:30%;background-color:#ffffcc;font-weight:bold;color:#000}
     .h{background:url('http://python.org/images/header-bg2.png') repeat-x;font-weight:bold;color:#000}
     .v{background-color:#f2f2f2;color:#000}
     .vr{background-color:#cccccc;text-align:right;color:#000}
     img{float:right;border:0px;}
     hr{width:600px;background-color:#ccc;border:0px;height:1px;color:#000}
    </style>
"""


class HtmlHelpers():
    def __init__(self, info):
        self.info = info

    def table(self, html):
        return '<table border="0" cellpadding="3" width="600">%s</table><br>' % html

    def maketable(self, data):
        html = ''
        for k in data:
            v = cgi.escape(str(data[k]))
            html += '<tr><td class="e">%s</td><td class="v">%s</td></tr>' % (k, v)
        return self.table(html)

    @property
    def section_title(self):
        html = '<tr class="h"><td>'
        html += '<a href="http://python.org/"><img border="0" src="http://python.org/images/python-logo.gif"></a>'
        html += '<h1 class="p">Python %s</h1>' % self.info['System information']['Python version']
        html += '</td></tr>'
        return self.table(html)

    @property
    def section_system(self):
        return self.maketable(self.info['System information'])

    @property
    def section_py_internals(self):
        return self.maketable(self.info['Python internals'])

    @property
    def section_os_internals(self):
        return self.maketable(self.info['OS internals'])

    @property
    def section_environ(self):
        return self.maketable(self.info['Environment variables'])

    @property
    def section_database(self):
        return self.maketable(self.info['Database support'])

    @property
    def section_compression(self):
        return self.maketable(self.info['Compression and archiving'])

    @property
    def section_ldap(self):
        if 'LDAP support' not in self.info:
            return ''

        html = self.maketable(self.info['LDAP support'])
        return '<h2>LDAP support</h2>' + html

    @property
    def section_socket(self):
        if 'Socket' not in self.info:
            return ''
        html = self.maketable(self.info['Socket'])
        return '<h2>Socket</h2>' + html

    @property
    def section_multimedia(self):
        return self.maketable(self.info['Multimedia support'])

    @property
    def section_copyright(self):
        text = self.info['Copyright'].replace('\n\n', '<br>') \
            .replace('\r\n', '<br />').replace('(c)', '&copy;')
        return self.table('<tr class="v"><td>%s</td></tr>' % text)


def render_html(info):
    """Return the HTML page version of pyinfo."""
    helper = HtmlHelpers(info)
    return PAGE_HTML.format(h=helper, style=PAGE_STYLE)


def _guess_max_lengths(info):
    """Compute the maximum length for keys and lines."""
    max_key_len = 0
    max_line_len = 0

    for sec in info:
        for k in info[sec]:
            l_k = len(k)
            l_data = len(info[sec])
            line_len = l_k + 2 + l_data
            if line_len > 80:
                # skip lines which wraps
                continue

            if line_len > max_line_len:
                # found a bigger line length
                max_line_len = line_len

            if l_k > max_key_len:
                # Found a bigger key length
                max_key_len = l_k

    return (max_key_len, max_line_len)


def render_text(info):
    """Create the text version of pyinfo."""
    d = []
    max_key_len, max_line_len = _guess_max_lengths(info)

    for sec in info:
        # build section title (2 = spaces for padding)
        title_size = len(sec)
        sym = "=" * int((max_line_len - title_size - 2) / 2)
        d += ["{} {} {}".format(sym, sec, sym)]

        sec_info = info[sec]

        if sec == 'Copyright':
            # customize the output for global section because of long copyright
            d += [sec_info]
            continue
        fmt = "{:>%s}: {}" % (max_key_len,)
        for k in sec_info:
            d += [fmt.format(k, sec_info[k])]
        d += [""]

    return "\n".join(d)



# -*- coding: utf-8 -*-
import os
import platform
import socket
import sys
from collections import OrderedDict

from . import renderers

__version__ = '1.0.1'

optional_modules_list = [
    'Cookie',
    'zlib', 'gzip', 'bz2', 'zipfile', 'tarfile',
    'ldap',
    'socket',
    'audioop', 'curses', 'imageop', 'aifc', 'sunau', 'wave', 'chunk',
    'colorsys', 'rgbimg', 'imghdr', 'sndhdr', 'ossaudiodev', 'sunaudiodev',
    'adodbapi', 'cx_Oracle', 'ibm_db', 'mxODBC', 'MySQLdb', 'pgdb', 'PyDO',
    'sapdbapi', 'sqlite3'
]
for i in optional_modules_list:
    try:
        module = __import__(i)
        sys.modules[i] = module
        globals()[i] = module
    except:
        pass

_PYTHON_MAJOR = sys.version_info[0]


def try_exec(fn):
    """Execute function and handles exceptions as error."""
    try:
        return fn()
    except Exception as e:
        return "Error: {}".format(repr(e))


def imported(module):
    if module in sys.modules:
        return 'enabled'
    return 'disabled'


def python_info():
    data = OrderedDict()
    data['System information'] = collect_system_info()
    data['Python internals'] = collect_py_internals()
    data['OS internals'] = collect_os_internals()
    data['Environment variables'] = collect_environ()
    data['Database support'] = collect_database()
    data['Compression and archiving'] = collect_compression()
    if 'ldap' in sys.modules:
        data['LDAP support'] = collect_ldap()

    if 'socket' in sys.modules:
        data['Socket'] = collect_socket_info()

    data['Multimedia support'] = collect_multimedia_info()
    data['Copyright'] = sys.copyright
    return data


def collect_system_info():
    """Return system information."""
    data = OrderedDict()
    data['Python version'] = platform.python_version()
    if hasattr(sys, 'subversion'):
        data['Python Subversion'] = ', '.join(sys.subversion)

    if platform.dist()[0] != '' and platform.dist()[1] != '':
        osversion = '{} {} ({} {})'.format(platform.system(),
                                           platform.release(),
                                           platform.dist()[0].capitalize(),
                                           platform.dist()[1])
    else:
        osversion = '{} {}'.format(platform.system(), platform.release())
    data['OS Version'] = osversion

    if hasattr(sys, 'executable'):
        data['Executable'] = sys.executable
    data['Build Date'] = platform.python_build()[1]
    data['Compiler'] = platform.python_compiler()
    if hasattr(sys, 'api_version'):
        data['Python API'] = sys.api_version
    return data


def collect_py_internals():
    """Return python internal informations."""
    data = OrderedDict()
    if hasattr(sys, 'builtin_module_names'):
        data['Built-in Modules'] = ', '.join(sys.builtin_module_names)
    data['Byte Order'] = sys.byteorder + ' endian'
    if hasattr(sys, 'getcheckinterval'):
        data['Check Interval'] = sys.getcheckinterval()
    if hasattr(sys, 'getfilesystemencoding'):
        data['File System Encoding'] = sys.getfilesystemencoding()
    if _PYTHON_MAJOR < 3:
        hex_maxint = str(hex(sys.maxint)).upper().replace("X", "x")
        data['Maximum Integer Size'] = '{} ({})'.format(sys.maxint, hex_maxint)
    if hasattr(sys, 'getrecursionlimit'):
        data['Maximum Recursion Depth'] = sys.getrecursionlimit()
    if hasattr(sys, 'tracebacklimit'):
        data['Maximum Traceback Limit'] = sys.tracebacklimit
    else:
        data['Maximum Traceback Limit'] = '1000'
    data['Maximum Unicode Code Point'] = sys.maxunicode
    return data


def collect_os_internals():
    data = OrderedDict()
    if hasattr(os, 'getcwd'):
        data['Current Working Directory'] = try_exec(os.getcwd)

    if hasattr(os, 'getegid'):
        data['Effective Group ID'] = try_exec(os.getegid)

    if hasattr(os, 'geteuid'):
        data['Effective User ID'] = try_exec(os.geteuid)

    if hasattr(os, 'getgid'):
        data['Group ID'] = try_exec(os.getgid)

    if hasattr(os, 'getgroups'):
        data['Group Membership'] = try_exec(lambda: ', '.join(map(str, os.getgroups())))

    if hasattr(os, 'linesep'):
        data['Line Seperator'] = repr(os.linesep)[1:-1]

    if hasattr(os, 'getloadavg'):
        data['Load Average'] = try_exec(lambda: ', '.join(map(str, map(lambda x: round(x, 2), os.getloadavg()))))

    if hasattr(os, 'pathsep'):
        data['Path Seperator'] = os.pathsep

    if hasattr(os, 'getpid') and hasattr(os, 'getppid'):
        data['Process ID'] = try_exec(lambda: '{} (parent: {})'.format(os.getpid(), os.getppid()))

    if hasattr(os, 'getuid'):
        data['User ID'] = try_exec(os.getuid)
    return data


def collect_environ():
    """Return environment vars."""
    envvars = os.environ.keys()
    if _PYTHON_MAJOR > 2:
        envvars = list(envvars)
    envvars.sort()
    data = OrderedDict()
    for envvar in envvars:
        data[envvar] = str(os.environ[envvar])
    return data


def collect_database():
    """Return database support information."""
    data = OrderedDict()
    data['DB2/Informix (ibm_db)'] = imported('ibm_db')
    data['MSSQL (adodbapi)'] = imported('adodbapi')
    data['MySQL (MySQL-Python)'] = imported('MySQLdb')
    data['ODBC (mxODBC)'] = imported('mxODBC')
    data['Oracle (cx_Oracle)'] = imported('cx_Oracle')
    data['PostgreSQL (PyGreSQL)'] = imported('pgdb')
    data['Python Data Objects (PyDO)'] = imported('PyDO')
    data['SAP DB (sapdbapi)'] = imported('sapdbapi')
    data['SQLite3'] = imported('sqlite3')
    return data


def collect_compression():
    data = OrderedDict()
    data['Bzip2 Support'] = imported('bz2')
    data['Gzip Support'] = imported('gzip')
    data['Tar Support'] = imported('tarfile')
    data['Zip Support'] = imported('zipfile')
    data['Zlib Support'] = imported('zlib')
    return data


def collect_ldap():
    """Return ldap module information."""
    data = OrderedDict()
    data['Python-LDAP Version'] = ldap.__version__
    data['API Version'] = ldap.API_VERSION
    data['Default Protocol Version'] = ldap.VERSION
    data['Minimum Protocol Version'] = ldap.VERSION_MIN
    data['Maximum Protocol Version'] = ldap.VERSION_MAX
    data['SASL Support (Cyrus-SASL)'] = ldap.SASL_AVAIL
    data['TLS Support (OpenSSL)'] = ldap.TLS_AVAIL
    data['Vendor Version'] = ldap.VENDOR_VERSION
    return data


def collect_socket_info():
    """Return information available from socket library."""
    data = OrderedDict()
    data['Hostname'] = try_exec(socket.gethostname)

    hostname = try_exec(lambda: socket.gethostbyaddr(socket.gethostname())[0])
    data['Hostname (fully qualified)'] = hostname

    ip_addr = try_exec(lambda: socket.gethostbyname(socket.gethostname()))
    data['IP Address'] = ip_addr

    data['IPv6 Support'] = getattr(socket, 'has_ipv6', False)
    data['SSL Support'] = hasattr(socket, 'ssl')
    return data


def collect_multimedia_info():
    """Return multimedia related information."""
    data = OrderedDict()
    data['AIFF Support'] = imported('aifc')
    data['Color System Conversion Support'] = ('colorsys')
    data['curses Support'] = imported('curses')
    data['IFF Chunk Support'] = imported('chunk')
    data['Image Header Support'] = imported('imghdr')
    data['OSS Audio Device Support'] = imported('ossaudiodev')
    data['Raw Audio Support'] = imported('audioop')
    data['Raw Image Support'] = imported('imageop')
    data['SGI RGB Support'] = imported('rgbimg')
    data['Sound Header Support'] = imported('sndhdr')
    data['Sun Audio Device Support'] = imported('sunaudiodev')
    data['Sun AU Support'] = imported('sunau')
    data['Wave Support'] = imported('wave')
    return data


def info_as_text():
    info = python_info()
    return renderers.render_text(info)


def info_as_html():
    info = python_info()
    return renderers.render_html(info)


def print_info():
    """Cli program main."""
    print(info_as_text())

if __name__ == '__main__':
    print_info()



# -*- coding: utf-8 -*-


def application(environ, start_response):
    import pyinfo

    output = pyinfo.info_as_html()

    start_response('200 OK', [('Content-type', 'text/html')])
    return [output.encode('utf-8')]



# -*- coding: utf-8 -*-
import codecs
import os
import re

from setuptools import setup


def read(*parts):
    here = os.path.abspath(os.path.dirname(__file__))
    return codecs.open(os.path.join(here, *parts)).read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='pyinfo',
    description='python environment info as phpinfo does with php',
    packages=['pyinfo'],
    version=find_version('pyinfo', '_pyinfo.py'),
    author="Vincenzo Farruggia",
    author_email="mastropinguino@networky.net",
    url='https://github.com/mastropinguino/pyinfo',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Terminals',
        'Topic :: Utilities'
    ],
    keywords=['python info', 'pyinfo', 'environment information'])
